/*
 * timelapse database for CYCLES
 * records data for timelapse photography
 * John F. Bono - 2013
 * built on code from Andrew Benson - 2008
 */

var sqlite = new SQLite;
var result = new SQLResult;
outlets = 2;

//open a file-based DB
function opendb(x)
{
	if (x) {
		sqlite.open(x,1);
		get_all();
	}
}

//close database
function closedb()
{
    sqlite.close();
}

//-------START database calls--------
//output complete database
function get_all()
{
    exec("SELECT * FROM TIMELAPSE");
}

//get the information of an entry formatted for the inspector patch
//with count output
function get_entry(){
	exec("SELECT * FROM TIMELAPSE WHERE event = '"+arguments[0]+"'");
	var numfields = result.numfields();
	for(var i=0; i<numfields; i++){
		outlet(1, "meta", result.fieldname(i),result.value(i,0));
	}
	outlet(1, "event",result.numrecords());
}

//------START database manipulation-----
//columns setup
function build_db()
{
     exec("CREATE TABLE IF NOT EXISTS TIMELAPSE(filePath VARCHAR(256),fileName VARCHAR(256))");
     add_column("acquiredMonth","acquiredDay","acquiredYear","acquiredHour","acquiredMiniute","acquiredSecond","event","camera","positionX","positionY","positionZ","rainfall","windSpeed","windDirection","barometricPressure","temperature","redLight","greenLight","blueLight","humidity","lunarCycle","tide");
}

//data entry
function insert_entry(){
    if(arguments.length>=12){
        exec("INSERT INTO TIMELAPSE ('filePath','fileName','acquiredMonth','acquiredDay','acquiredYear','acquiredHour','acquiredMiniute','acquiredSecond','event','camera','positionX','positionY','positionZ','rainfall','windSpeed','windDirection','barometricPressure','temperature','redLight','greenLight','blueLight','humidity','lunarCycle','tide') VALUES ('"+arguments[0]+"','"+arguments[1]+"','"+arguments[2]+"','"+arguments[3]+"','"+arguments[4]+"','"+arguments[5]+"','"+arguments[6]+"','"+arguments[7]+"','"+arguments[8]+"','"+arguments[9]+"','"+arguments[10]+"','"+arguments[11]+"','"+arguments[12]+"','"+arguments[13]+"','"+arguments[14]+"','"+arguments[15]+"','"+arguments[16]+"','"+arguments[17]+"','"+arguments[18]+"','"+arguments[19]+"','"+arguments[20]+"','"+arguments[21]+"','"+arguments[22]+"','"+arguments[23]+"')");
    }
    else{
        post("Wrong number of arguments");
    }
}

//remove all entries from the table and the umenu
function clearAll(){
    exec("DELETE FROM TIMELAPSE",result);
    outlet(1, "menu", "clear");
    post("All entries have been removed");
}

//change the value of a field in the database (name,field,value)
function change_entry(){
    if(arguments.length==3){
        exec("UPDATE TIMELAPSE SET '"+arguments[1]+"' = '"+arguments[2]+"' WHERE name = '"+arguments[0]+"'");
        get_all();
    }
    else{
        post("Wrong number of arguments");
    }
}

//add a new column to the table
function add_column(){
	for(i=0;i<arguments.length;i++){
	    exec("ALTER TABLE TIMELAPSE ADD '"+arguments[i]+"' varchar");
	}
}

function exec(arg)
{
	// execute the SQL statement in arg, returning results in the 'result' object
	sqlite.exec(arg, result);

	// access information about the returned records by calling functions on the result object
	formatResultForCellblock();
}

// format output for jitter.cellblock and output
function formatResultForCellblock()
{
    var numfields = result.numfields();
    var numrecords = result.numrecords();
    var fieldnames = new Array(numfields);
    var values = new Array(numfields);

	outlet(0, "clear", "all");
	outlet(0, "cols", numfields);
	outlet(0, "rows", numrecords + 1);    // rows +1 so we can create a header row

	for(var i=0; i<numfields; i++)
		outlet(0, "set", i, 0, result.fieldname(i));

	for(var i=0; i<numrecords; i++){
	    	for(var j=0; j<numfields; j++){
			        outlet(0, "set", j, i+1, result.value(j, i));
    	    }
    }
}